@@ -24,19 +24,11 @@ from pipenv.patched.pip._internal.req.constructors import (
 from pipenv.patched.pip._internal.req.req_file import parse_requirements
 from pipenv.patched.pip._internal.req.req_install import InstallRequirement
 from pipenv.patched.pip._internal.utils.temp_dir import global_tempdir_manager
-from pipenv.patched.pip._vendor import pkg_resources, rich
 from pipenv.patched.pip._vendor.packaging.utils import canonicalize_name
 from pipenv.project import Project
+from pipenv.utils import console, err
 from pipenv.utils.fileutils import create_tracked_tempdir
 from pipenv.utils.requirements import normalize_name
-from pipenv.vendor import click
-
-try:
-    # this is only in Python3.8 and later
-    from functools import cached_property
-except ImportError:
-    # eventually distlib will remove cached property when they drop Python3.7
-    from pipenv.patched.pip._vendor.distlib.util import cached_property
 
 from .dependencies import (
     HackedPythonVersion,
@@ -53,8 +45,10 @@ from .internet import is_pypi_url
 from .locking import format_requirement_for_lockfile, prepare_lockfile
 from .shell import make_posix, subprocess_run, temp_environ
 
-console = rich.console.Console()
-err = rich.console.Console(stderr=True)
+if sys.version_info < (3, 10):
+    from pipenv.vendor import importlib_metadata
+else:
+    import importlib.metadata as importlib_metadata
 
 
 def get_package_finder(
@@ -87,7 +81,6 @@ def get_package_finder(
 
 
 class HashCacheMixin:
-
     """Caches hashes of PyPI artifacts so we do not need to re-download them.
 
     Hashes are only cached when the URL appears to contain a hash in it and the
@@ -228,6 +221,7 @@ class Resolver:
                 markers_lookup[package_name] = install_req.markers
             if is_constraint:
                 constraints.add(dep)
+        # raise Exception(constraints, original_deps, install_reqs, pipfile_entries)
         resolver = Resolver(
             set(),
             req_dir,
@@ -267,6 +261,10 @@ class Resolver:
         if self.pre:
             pip_args.append("--pre")
         pip_args.extend(["--cache-dir", self.project.s.PIPENV_CACHE_DIR])
+        extra_pip_args = os.environ.get("PIPENV_EXTRA_PIP_ARGS")
+        if extra_pip_args:
+            extra_pip_args = json.loads(extra_pip_args)
+            pip_args.extend(extra_pip_args)
         return pip_args
 
     @property  # cached_property breaks authenticated private indexes
@@ -287,10 +285,6 @@ class Resolver:
         return constraint_filename
 
     @property
-    def constraint_file(self):
-        return self.prepare_constraint_file()
-
-    @cached_property
     def default_constraint_file(self):
         default_constraints = get_constraints_from_deps(self.project.packages)
         default_constraint_filename = prepare_constraint_file(
@@ -329,7 +323,7 @@ class Resolver:
                 alt_index_lookup[req_name] = index_mapping[index]
         return alt_index_lookup
 
-    @cached_property
+    @property
     def package_finder(self):
         finder = get_package_finder(
             install_cmd=self.pip_command,
@@ -347,18 +341,18 @@ class Resolver:
         finder._ignore_compatibility = ignore_compatibility
         return finder
 
-    @cached_property
+    @property
     def parsed_constraints(self):
         pip_options = self.pip_options
         pip_options.extra_index_urls = []
         return parse_requirements(
-            self.constraint_file,
+            self.prepare_constraint_file(),
             finder=self.finder(),
             session=self.session,
             options=pip_options,
         )
 
-    @cached_property
+    @property
     def parsed_default_constraints(self):
         pip_options = self.pip_options
         pip_options.extra_index_urls = []
@@ -371,7 +365,7 @@ class Resolver:
         )
         return set(parsed_default_constraints)
 
-    @cached_property
+    @property
     def default_constraints(self):
         possible_default_constraints = [
             install_req_from_parsed_requirement(
@@ -530,11 +524,10 @@ class Resolver:
                             if result.req:
                                 result.req.marker = marker
                         except TypeError as e:
-                            click.echo(
+                            err.print(
                                 f"Error generating python marker for {candidate}.  "
                                 f"Is the specifier {requires_python} incorrectly quoted or otherwise wrong?"
                                 f"Full error: {e}",
-                                err=True,
                             )
             new_tree.add(result)
 
@@ -588,7 +581,7 @@ class Resolver:
             return {self.project.get_hash_from_link(self.hash_cache, link)}
         return set()
 
-    @cached_property
+    @property
     def resolve_hashes(self):
         if self.results is not None:
             for ireq in self.results:
@@ -705,15 +698,14 @@ def actually_resolve_deps(
 
 def resolve(cmd, st, project):
     from pipenv.cmdparse import Script
-    from pipenv.vendor.click import echo
 
     c = subprocess_run(Script.parse(cmd).cmd_args, block=False, env=os.environ.copy())
     is_verbose = project.s.is_verbose()
-    err = ""
+    errors = ""
     for line in iter(c.stderr.readline, ""):
         if not line.rstrip():
             continue
-        err += line
+        errors += line
         if is_verbose:
             st.console.print(line.rstrip())
 
@@ -722,13 +714,13 @@ def resolve(cmd, st, project):
     out = c.stdout.read()
     if returncode != 0:
         st.console.print(environments.PIPENV_SPINNER_FAIL_TEXT.format("Locking Failed!"))
-        echo(out.strip(), err=True)
+        err.print(out.strip())
         if not is_verbose:
-            echo(err, err=True)
+            err.print(err)
         raise RuntimeError("Failed to lock Pipfile.lock!")
     if is_verbose:
-        echo(out.strip(), err=True)
-    return subprocess.CompletedProcess(c.args, returncode, out, err)
+        err.print(out.strip())
+    return subprocess.CompletedProcess(c.args, returncode, out, errors)
 
 
 def venv_resolve_deps(
@@ -743,6 +735,7 @@ def venv_resolve_deps(
     pipfile=None,
     lockfile=None,
     old_lock_data=None,
+    extra_pip_args=None,
 ):
     """
     Resolve dependencies for a pipenv project, acts as a portal to the target environment.
@@ -771,10 +764,10 @@ def venv_resolve_deps(
 
     if not deps:
         if not project.pipfile_exists:
-            return None
+            return {}
         deps = project.parsed_pipfile.get(category, {})
     if not deps:
-        return None
+        return {}
 
     if not pipfile:
         pipfile = getattr(project, category, {})
@@ -794,7 +787,11 @@ def venv_resolve_deps(
             os.environ["PIPENV_SITE_DIR"] = pipenv_site_dir
         else:
             os.environ.pop("PIPENV_SITE_DIR", None)
-        with console.status("Locking...", spinner=project.s.PIPENV_SPINNER) as st:
+        if extra_pip_args:
+            os.environ["PIPENV_EXTRA_PIP_ARGS"] = json.dumps(extra_pip_args)
+        with console.status(
+            f"Locking {category}...", spinner=project.s.PIPENV_SPINNER
+        ) as st:
             # This conversion is somewhat slow on local and file-type requirements since
             # we now download those requirements / make temporary folders to perform
             # dependency resolution on them, so we are including this step inside the
@@ -862,8 +859,8 @@ def venv_resolve_deps(
                         with open(target_file.name) as fh:
                             results = json.load(fh)
                     except (IndexError, json.JSONDecodeError):
-                        click.echo(c.stdout.strip(), err=True)
-                        click.echo(c.stderr.strip(), err=True)
+                        err.print(c.stdout.strip())
+                        err.print(c.stderr.strip())
                         if os.path.exists(target_file.name):
                             os.unlink(target_file.name)
                         raise RuntimeError("There was a problem with locking.")
@@ -873,13 +870,15 @@ def venv_resolve_deps(
                         environments.PIPENV_SPINNER_OK_TEXT.format("Success!")
                     )
                     if not project.s.is_verbose() and c.stderr.strip():
-                        click.echo(click.style(f"Warning: {c.stderr.strip()}"), err=True)
+                        err.print(
+                            f"Warning: {c.stderr.strip()}", overflow="ignore", crop=False
+                        )
                 else:
                     st.console.print(
                         environments.PIPENV_SPINNER_FAIL_TEXT.format("Locking Failed!")
                     )
-                    click.echo(f"Output: {c.stdout.strip()}", err=True)
-                    click.echo(f"Error: {c.stderr.strip()}", err=True)
+                    err.print(f"Output: {c.stdout.strip()}")
+                    err.print(f"Error: {c.stderr.strip()}")
     if lockfile_section not in lockfile:
         lockfile[lockfile_section] = {}
     return prepare_lockfile(
@@ -959,9 +958,7 @@ def resolve_deps(
 
 @lru_cache
 def get_pipenv_sitedir() -> Optional[str]:
-    site_dir = next(
-        iter(d for d in pkg_resources.working_set if d.key.lower() == "pipenv"), None
-    )
-    if site_dir is not None:
-        return site_dir.location
+    for dist in importlib_metadata.distributions():
+        if dist.metadata.get("Name", "").lower() == "pipenv":
+            return str(dist.locate_file(""))
     return None
